#ifndef DATASET_H
#define DATASET_H

#include <QObject>

/*!
Contains info for layouting
*/
struct LayoutInfo
{
	QPainterPath path;
	QBrush brush;
};


typedef float t_entry;
typedef QVector<t_entry> EntryVector;

/*!
Represents a single layer of the stacked graph.
Includes the data for each time point (\c data), label and other information which is calculated by various steps in the graphing pipeline.
*/
struct DataItem
{
	/*!
	A user-presentable label of the data
	*/
	QString label;
	/*!
	The data values of this layer, for each time point
	*/
	EntryVector data;
	/*!
	The color for drawing this layer
	*/
	QColor color;
	/*!
	Information created during layouting
	*/
	LayoutInfo layout;
	/*!
	The optimal point to place the label, as determined during layouting
	*/
	QPointF labelPos;

	/*!
	The first time point where \ref data has a value
	*/
	int onsetTime;
	/*!
	The last time point where \ref data has a value
	*/
	int lastTime;
	/*!
	The maximum data value change in this layer (used for sorting/coloring)
	*/
	t_entry volatility;
	/*!
	The sum of all the data values in this layer.
	*/
	t_entry sum;
	/*!
	The maximum data value in this layer
	*/
	t_entry max;
	/*!
	The index of the maximum data value in the \ref data vector
	*/
	int maxIdx;
	/*!
	The index of this \ref DataItem in the parent \ref DataSet (as per \ref DataSet::getData)
	*/
	int idx;
};


typedef QList<DataItem> DataList;
typedef QList<DataItem*> DataPtrList;
typedef QList<DataItem*>& DataPtrListRef;
typedef DataList& DataListRef;

/*!
The data set. Data entries are stored as a list of \ref DataItem objects.
*/
struct DataSet
{
public:
	
	/*!
	Returns a reference to the internal data set.
	*/
	DataListRef getData() {return mData;}
	
	/*!
	Returns a list of pointers which is sorted according to the current sorting algorithm.
	This is only valid after the sorting algorithm was called, otherwise it may contain old or undefined data.
	*/
	DataPtrListRef getSortedData() {return mSortedData;}
	
	/*!
	Returns the cumulated height at each point of the timeseries
	*/
	QVector<float>& getCumHeight() { return mCumHeight; }
	
	/*!
	Retuns the size of the dataset in X direction, i.e. the length of the time series (usually number of months, 12)
	*/
	int getSizeX() const {return mSizeX;}
	
	/*!
	Returns the size of the dataset in Y direction, which is the number of layers this dataset has
	*/
	int getSizeY() const {return mData.size();}
	
	/*!
	Returns the minimum onsettime of the overall dataset
	*/
	int getMinOnsetTime() const {return minOnsetTime;}
	
	/*!
	Returns the maximum onsettime of the overall dataset
	*/
	int getMaxOnsetTime() const {return maxOnsetTime;}
	
	/*!
	Returns the minimum volatility of the overall dataset
	*/
	float getMinVolatility() const {return minVolatility;}
	
	/*!
	Returns the maximum volatility of the overall dataset
	*/
	float getMaxVolatility() const {return maxVolatility;}

	/*!
	Returns the minimum sum of the overall dataset
	*/
	float getMinSum() const { return minSum;}
	
	/*!
	Returns the minimum sum of the overall dataset
	*/
	float getMaxSum() const { return maxSum;}

	/*!
	The path to the file where the dataset was loaded from
	*/
	QString getPath() const { return mPath; }
	
	/*!
	Resets the internal sorted list to the unsorted state, i.e. the same order it appears in the normal data list.
	*/
	void setUnsorted() 
	{
		mSortedData.clear();
		auto it = mData.begin();
		while(it!=mData.end())
		{
			mSortedData.append(&(*it++));
		}
	}
	/*!
	Returns true if the internal data set is empty, otherwise false
	*/
	bool isEmpty() const { return mData.isEmpty(); }
private:
	int mSizeX;
	QString mPath;
	DataList mData;
	DataPtrList mSortedData;
	QVector<float> mCumHeight;
	int minOnsetTime;
	int maxOnsetTime;
	float minVolatility;
	float maxVolatility;
	float minSum;
	float maxSum;

	friend class DataLoader;
};

#endif // DATASET_H
