#include "stdafx.h"
#include "fileloader.h"

#define RND_FLOAT ((float)qrand() / (float)RAND_MAX)
//random integer, a/b inclusive
#define RND_INT(a,b) qRound( RND_FLOAT * (b-a)) + a

DataLoader::DataLoader(QObject *parent)
	: QObject(parent)
{
	qsrand(1234);
}

DataLoader::~DataLoader()
{
}

bool DataLoader::LoadData(const QString& path, DataSet& dataset) const
{
	qDebug()<<"Loading file: " << path;

	QFile file (path);
	if (file.open(QIODevice::ReadOnly)) {

		QMap<QString,DataItem> map;
		QString data = file.readAll();

		QTextStream textStream(&data);
		QStringList entries;
		QStringList lines;
		QString line;

		//check if file valid
		line = textStream.readLine();
		if(line.isNull() || line.compare(FILE_HEADER,Qt::CaseSensitive) != 0){
			return false;
		}

		while (!textStream.atEnd()) {
			line = textStream.readLine();
			lines.append(line);
		}

		int timeCnt = lines.length();
		dataset.mPath = path;
		dataset.mSizeX = timeCnt;

		for(int i = 0;i<lines.length();i++){
			line = lines.at(i);

			entries = line.split(QChar(';'),QString::SplitBehavior::KeepEmptyParts);

			for(int j=0;j<entries.length();j++){
				QString entry = entries[j];
				entry = entry.mid(1,entry.length()-2);

				QStringList keyvalue = entry.split(QChar(','),QString::SplitBehavior::KeepEmptyParts);

				QString key = keyvalue.first();

				bool ok;
				t_entry value = keyvalue.last().toFloat(&ok);

				if(!ok){
					value = -1;
				}

				if(!map.contains(key)){
					DataItem item;
					item.label = key;
					item.onsetTime = i;
					item.volatility = 0;
					item.sum = 0;
					item.max = 0;
					item.maxIdx = -1;
					item.data.fill(0.0f,timeCnt);
					item.layout = LayoutInfo();
					map.insert(key, item);
				}
				if(map[key].max < value)
				{
					map[key].max = value;
					map[key].maxIdx = i;
				}
				if(value>0)
				{
					map[key].lastTime = i;
				}
				map[key].data[i] = value;
				map[key].sum += value;

				if (i > 0) 
				{
					map[key].volatility = map[key].volatility > abs(value - map[key].data[i-1]) ? map[key].volatility : abs(value - map[key].data[i-1]);
				}
			}
		}

		dataset.mData = map.values();

		//calculate min/max values in a different loop
		this->setMinMaxValues(dataset);
	}

	return true;
}

void DataLoader::RandomData(int numItems, DataSet& dataset) const
{
	//12 months
	dataset.mSizeX = 12;

	DataList data;

	for(int i =0;i<numItems;++i)
	{
		//generate random data
		DataItem item;
		item.layout = LayoutInfo();
		item.label = QString("Layer #%1").arg(QString::number(i));
		createRandomItemData(item);

		data.append(item);
	}

	dataset.mPath = "<random data>";
	dataset.mData = data;

	this->setMinMaxValues(dataset);
}

void DataLoader::createRandomItemData(DataItem& item) const
{
	const int datalength = 12;
	item.data.fill(0,datalength);
	item.sum = 0;
	item.volatility = 0;
	item.max = 0;
	item.maxIdx = -1;
	item.onsetTime = -1;
	item.lastTime = datalength-1;


	//the data consists of 4 +- 1 gaussian bumps
	int maxBumps = RND_INT(3,5);
	for(int j = 0;j<maxBumps;++j)
	{
		float height = (RND_FLOAT * 20) * 1000000;
		float cx = (float)(2.0f * RND_FLOAT - 0.5f);
		float r = RND_FLOAT / 10.0f;

		for(int i = 0;i<datalength;++i)
		{
			//evaluate gaussian
			double a = (i / (double)datalength - cx) / r;
			double val =( height * qExp( - a * a) );
			item.data[i]+= val;
		}
	}

	//create sum + volatility, remove small values
	for(int i = 0;i<datalength;++i)
	{
		t_entry val = item.data[i];
		if(val < 100000) //100k is rather small for our graphs
		{
			item.data[i] = 0;
			continue;
		}
		if(item.onsetTime == -1)
		{
			item.onsetTime = i;
		}
		if(val > item.max)
		{
			item.max = val;
			item.maxIdx = i;
		}

		item.sum+= val;

		if (i > 0) {
			item.volatility = item.volatility > abs( val -  item.data[i-1]) ?item.volatility : abs(val - item.data[i-1]);
		}
		item.lastTime = i;
	}

	while(item.onsetTime==-1)
	{
		//through unfortunate luck, the item may have no data at all
		//repeat until it has at least something, or the graph layouter runs into problems
		createRandomItemData(item);
	}
}

void DataLoader::setMinMaxValues(DataSet& dataset) const
{
	dataset.minOnsetTime = INT_MAX;
	dataset.maxOnsetTime = INT_MIN;
	dataset.minVolatility = FLT_MAX;
	dataset.maxVolatility = FLT_MIN;
	dataset.minSum = FLT_MAX;
	dataset.maxSum = FLT_MIN;

	int cnt = 0;
	const DataListRef dat = dataset.getData();
	for (auto it = dat.begin(); it != dat.end(); it++)
	{
		DataItem& item = *it;
		dataset.minOnsetTime = item.onsetTime < dataset.minOnsetTime ? item.onsetTime : dataset.minOnsetTime;
		dataset.maxOnsetTime = item.onsetTime > dataset.maxOnsetTime ? item.onsetTime : dataset.maxOnsetTime;
		dataset.minVolatility = item.volatility < dataset.minVolatility ? item.volatility : dataset.minVolatility;
		dataset.maxVolatility = item.volatility > dataset.maxVolatility ? item.volatility : dataset.maxVolatility;
		dataset.minSum = item.sum < dataset.minSum ? item.sum : dataset.minSum;
		dataset.maxSum = item.sum > dataset.maxSum ? item.sum : dataset.maxSum;

		item.idx = cnt;

		cnt++;
	}
}