#include "stdafx.h"
#include "graphscene.h"

#include "stackgraphlayouter.h"
#include "themerivergraphlayouter.h"
#include "minwigglegraphlayouter.h"
#include "streamgraphlayouter.h"

#include "ontopgraphsorter.h"
#include "insideoutgraphsorter.h"
#include "weightedgraphsorter.h"

#include "graphcolorizer.h"

// THIS IS NOT PIXEL SIZE!
// but rather the extents of the graph scene
// chosen so that it is convinient to work with
#define RANGE_X 1024
#define RANGE_Y 512

/*!
A subclass of \ref QGraphicsPathItem, mainly used to hide the selection rectangle.
*/
class GraphItem :  public QGraphicsPathItem
{
public:
	GraphItem(const QPainterPath& path, QGraphicsItem* parent = 0) 
		: QGraphicsPathItem(path,parent)
	{
	}

	// to remove the selection rectangle, we actually have to override the item
	// how inconvinient
	// http://stackoverflow.com/questions/10985028/how-to-remove-border-around-qgraphicsitem-when-selected
	void paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget = 0) override
	{
		QStyleOptionGraphicsItem myOption(*option);
		myOption.state &= ~QStyle::State_Selected;
		QGraphicsPathItem::paint(painter, &myOption, widget);
	}

protected:
	void mousePressEvent(QGraphicsSceneMouseEvent* ev) override
	{

		//ev->ignore();
		//ev->accept();
	}
};

GraphScene::GraphScene(QWidget *parent)
	: QGraphicsScene(parent)
{
	// create list of layouters and sorters available
	mLayouters.append(new StackGraphLayouter(this));
	mLayouters.append(new ThemeRiverGraphLayouter(this));
	mLayouters.append(new MinWiggleGraphLayouter(this));
	mLayouters.append(new StreamGraphLayouter(this));

	mSorters.append(new OnTopGraphSorter(this));
	mSorters.append(new InsideOutGraphSorter(this));
	mSorters.append(new WeightedGraphSorter(this));

	mCurrentLayouter = mLayouters.at(0);
	mCurrentSorter = nullptr; //default unsorted

	mCurrentColorizer = new GraphColorizer(this);
	mDoLabeling = true; //by default labeling is active

	QFont f("Monospace");
	f.setStyleHint(QFont::Monospace);
	//f.setStyleHint(QFont::TypeWriter);
	QToolTip::setFont(f);

	//weird to connect this to base class, but seems to be no other way to get selections
	connect(this,&QGraphicsScene::selectionChanged,this, &GraphScene::onSelectionChanged);
}

GraphScene::~GraphScene()
{
	//explicit disconnect, or we would get a signal from the base class destructor, while the current class has been destroyed! 
	//bad things happen then
	disconnect(this,&QGraphicsScene::selectionChanged,this, &GraphScene::onSelectionChanged);
}

void GraphScene::mousePressEvent(QGraphicsSceneMouseEvent* ev) 
{
	QGraphicsScene::mousePressEvent(ev);
	ev->setAccepted(false);
}

void GraphScene::SetLayout(GraphLayouter* const layouter)
{
	this->mCurrentLayouter = layouter;
	emit redrawRequired();
}

void GraphScene::SetSorter(GraphSorter* const sorter)
{
	this->mCurrentSorter = sorter;
	emit redrawRequired();
}

void GraphScene::SetInterpolationMode(GraphLayouter::InterpolationMode const interpMode)
{
	this->mCurrentInterpMode = interpMode;
	emit redrawRequired();
}

void GraphScene::SetSortingAttribute(GraphSorter::SortingAttribute const sortAttribute)
{
	this->mCurrentSortAttribute = sortAttribute;
	emit redrawRequired();
}

void GraphScene::SetColoringAttribute(GraphColorizer::ColoringAttribute const coloringAttribute)
{
	this->mCurrentColoringAttribute = coloringAttribute;
	emit redrawRequired();
}

void GraphScene::SetColors(QColor const beginColor, QColor const endColor)
{
	this->mCurrentColorizer->SetColors(beginColor, endColor);
	emit redrawRequired();
}

void GraphScene::Set2DColoring(bool enable)
{
	this->mCurrentColorizer->Set2DColoring(enable);
	emit redrawRequired();
}

void GraphScene::SetLabeling(bool enabled)
{
	mDoLabeling = enabled;
	emit redrawRequired();
}

void GraphScene::calculateScale(DataSet& set, double& scaleX, double& scaleY)
{
	EntryVector& cumHeight = set.getCumHeight();
	cumHeight.fill(0.0,set.getSizeX());

	double maxHeight = DBL_MIN;

	const DataListRef dat = set.getData();
	for (auto it = dat.begin(); it != dat.end(); it++)
	{
		DataItem& item = *it;
		EntryVector& timeSeries = item.data;

		for (int i = 0; i < timeSeries.size(); i++) 
		{
			cumHeight[i] += timeSeries.at(i);

			if (maxHeight < cumHeight.at(i)) {
				maxHeight = cumHeight.at(i);
			}
		}
	}

	scaleX = RANGE_X / (double)(set.getSizeX()-1.0);
	scaleY = RANGE_Y / maxHeight;
}

void GraphScene::CreateStackedGraph(DataSet& dataset)
{
	if(dataset.isEmpty()){
		return;
	}

	//remove old selection value, important to do this before clear
	this->mOldSelection.clear();

	//clear current image
	this->clear();

	double scaleX, scaleY;
	//this also sets cumheight
	this->calculateScale(dataset,scaleX, scaleY);

	//remove current sorting
	dataset.setUnsorted();

	//sort the data
	if(mCurrentSorter != nullptr)
	{
		mCurrentSorter->Sort(dataset, mCurrentSortAttribute);
	}

	//layout the data
	mCurrentLayouter->Layout(dataset, mCurrentInterpMode, scaleX,scaleY);

	//assign colors to the layers
	mCurrentColorizer->Colorize(dataset, mCurrentColoringAttribute);

	QPen pen(Qt::black,0.3);

	{
		const DataPtrListRef dat = dataset.getSortedData();
		for (auto it = dat.cbegin(); it != dat.cend(); it++)
		{
			DataItem* item = *it;

			GraphItem* gi = new GraphItem(item->layout.path);
			gi->setPen(pen);
			gi->setBrush(QBrush(item->color));
			addItem(gi);

			//let selections happen
			gi->setFlag(QGraphicsItem::ItemIsSelectable);
			//set the first data to the index of the corresponding data element
			gi->setData(0,item->idx);

			if(mDoLabeling)
				placeLabel(item->label,item->layout.path,item->labelPos,QBrush(QColor(0,0,0,150)));
		}
	}

	//update scene rect to be the bounding box
	this->setSceneRect(this->itemsBoundingRect());

	mCurrentData = dataset;
}

void GraphScene::placeLabel(const QString& text, const QPainterPath& path, const QPointF& optimalPoint, const QBrush& brush)
{
	//draw label
	//brute force positioning
	const int MAX_FONTSIZE = 12;
	const int MIN_FONTSIZE = 6;
	const int MAX_STRINGLENGTH = 15;
	const int SEARCH_INTERVAL = 5;
	const int SEARCH_MAX = 16;

	const int SEARCH_MID = SEARCH_MAX / 2;


	QString str = text;
	if(str.length()>MAX_STRINGLENGTH)
	{
		str = str.left(MAX_STRINGLENGTH-3);
		str.append("...");
	}
	QGraphicsSimpleTextItem* label = new QGraphicsSimpleTextItem(str);

	bool success = false;

	for(int fontsize = MAX_FONTSIZE;fontsize>=MIN_FONTSIZE;--fontsize)
	{
		label->setFont(QFont("Tahoma",fontsize));
		//create bounding rect
		auto rect = label->boundingRect();
		float width = rect.width();
		float height = rect.height();

		QPointF pos = optimalPoint;
		for(int i = 0;i<SEARCH_MAX/2;++i)
		{
			float xPos;
			//start at best pos, go right and then left
			if(i < SEARCH_MID/2)
			{
				xPos = pos.x() + SEARCH_INTERVAL/2.0f * i;
			}
			else
			{
				xPos = pos.x() - (SEARCH_INTERVAL/2.0f - SEARCH_MID/2.0f) * i;
			}
			for(int j = 0;j<SEARCH_MAX;++j)
			{
				float yPos;
				//start at best pos, go down and then up
				if(j < SEARCH_MID)
				{
					yPos = pos.y() + SEARCH_INTERVAL * j;
				}
				else
				{
					yPos = pos.y() - (SEARCH_INTERVAL - SEARCH_MID) * j;
				}

				//calc topleft
				QPointF tl = QPointF(xPos - width/2,yPos - height/2);

				//translate rect
				QRectF rect2 = rect;
				rect2.translate(tl);

				if(path.contains(rect2))
				{
					//finished!
					label->setPos(tl);
					label->setBrush(brush);
					addItem(label);
					success = true;
					break;
				}
			}
			if(success)
				break;
		}
		if(success)
			break;
	}

	if(!success)
	{
		//no suitable place found, delete the label!
		delete label;
	}
}


QImage GraphScene::GetBitmap(int sizex, int sizey, bool keepAspect)
{
	//create img with alpha and clear
	QImage img(sizex,sizey,QImage::Format_ARGB32);
	img.fill(Qt::GlobalColor::transparent);

	//create antialiased painter
	QPainter painter(&img);
	painter.setRenderHint(QPainter::Antialiasing);

	//render the scene, ignoring aspect ratio
	this->render(&painter,QRectF(0,0,sizex,sizey),this->sceneRect(), keepAspect? Qt::KeepAspectRatio : Qt::IgnoreAspectRatio);

	//QImage is implicitly shared, so this is no problem
	return img;
}

QString formatMoney(int val)
{
	QString suffix=" ";

	float fVal = val;
	if(val > pow(10,6))
	{
		fVal /=  pow(10,6);
		suffix = "M";
	}
	else if (val > pow(10,3))
	{
		fVal /=  pow(10,3);
		suffix = "K";
	}

	QString str = QString("$ %1 %2").arg(fVal,7,'f',3).arg(suffix);
	//qDebug()<<str;
	return str;
}

QString GraphScene::getPopupString(const DataItem& item)
{
	QString ret = item.label;
	ret.append("\n\n");
	ret.append("Month\t\tData\n");

	for(int i =0;i<item.data.size();++i)
	{
		ret.append( QDate::fromString(QString::number(i+1),"M").toString("MMM") );
		ret.append("\t\t");
		ret.append(formatMoney(item.data.at(i)));
		ret.append("\n");
	}
	ret.append("\n");
	ret.append("Total:\t\t");
	ret.append(formatMoney(item.sum));

	return ret;
}

void GraphScene::onSelectionChanged()
{
	//reset old selection
	for(QGraphicsItem*& item : mOldSelection)
	{
		auto pi = dynamic_cast<GraphItem*> (item);
		int idx = item->data(0).toInt();
		const DataItem& di = mCurrentData.getData().at(item->data(0).toInt());
		pi->setBrush(QBrush(di.color));
	}

	auto newSelection = this->selectedItems();
	for(QGraphicsItem*& item : newSelection)
	{
		auto pi = dynamic_cast<GraphItem*> (item);
		int idx = item->data(0).toInt();
		const DataItem& di = mCurrentData.getData().at(item->data(0).toInt());
		pi->setBrush(QBrush(di.color.lighter()));
		QToolTip::showText(QCursor::pos(),getPopupString(di),this->views().at(0));
	}

	mOldSelection = newSelection;
}