#ifndef GRAPHSCENE_H
#define GRAPHSCENE_H

#include "graphlayouter.h"
#include "graphsorter.h"
#include "graphcolorizer.h"
#include <QGraphicsScene>

/*!
A QGraphicsScene subclass which is responsible for the creation, drawing and rendering of a stacked graph.
Call \ref CreateStackedGraph to create the graph with the currently set attributes.
\ref GetBitmap can be used for exporting.
*/
class GraphScene : public QGraphicsScene
{
	Q_OBJECT

public:
	explicit GraphScene(QWidget *parent);
	~GraphScene();

	/*!
	Creates a stacked graph out of the given dataset.
	This function puts together the results of the current \ref GraphSorter, \ref GraphLabeler and \ref GraphColorizer to create a visual representation of the data set.
	Labelling is also optionally performed using a brute-force algorithm.
	\param data the DataSet to use.
	*/
	void CreateStackedGraph(DataSet& data);

	/*!
	Returns the available layouts.
	*/
	QVector<GraphLayouter*> GetLayouts() const  { return mLayouters; }
	/*!
	Returns the available sorters
	*/
	QVector<GraphSorter*> GetSorters() const { return mSorters; }

	/*!
	Sets the layouter to use in the \ref CreateStackedGraph method
	\param layout the layouter
	*/
	void SetLayout(GraphLayouter* const layouter);

	/*!
	Sets the sorter to use in the \ref CreateStackedGraph method
	\param sorter the sorter
	*/
	void SetSorter(GraphSorter* const sorter);

	/*!
	Sets the interpolation method to be used by the \ref GraphLayouter when layouting the graph
	\param interpMode the interpolation mode
	*/
	void SetInterpolationMode(GraphLayouter::InterpolationMode const interpMode);

	/*!
	Sets the sorting attribute to be used by the \ref GraphSorter when sorting the dataset
	\param sortAttribute the attribute used to sort the dataset
	*/
	void SetSortingAttribute(GraphSorter::SortingAttribute const sortAttribute);

	/*!
	Sets the coloring attribute to be used by the \ref GraphColorizer when assigning colors to the graph layers
	\param colorAttribute the attribute used to calculate appropriate colors
	*/
	void SetColoringAttribute(GraphColorizer::ColoringAttribute const colorAttribute);

	/*!
	Determines if the layers should show their labels. The algorithm used for sizing and positioning the labels is based on a simple brute-force method,
	and may not give the best results in most cases. If no label can fit in a given layer, it gets no label at all.
	\param enabled if true, draw labels
	*/
	void SetLabeling(bool enabled);

	/*!
	Sets the colors which define the start and end color of the linearly interpolated color gradient used to define the stacked graphs layer colors	
	\param color the color to set
	*/
	void SetColors(QColor const beginColor, QColor const endColor);

	/*!
	Sets a flag which enables 2D coloring: alpha value of color changes according to the layours sum value
	*/
	void Set2DColoring(bool enable);

	/*!
	Renders the current graph into a \c QImage.
	\param width The pixel-width of the image
	\param heigth The pixel-height of the image
	\param keepAspect If true, the aspect ratio of the graph is kept as it is in the application. If false, it uses the ratio of the image.
	\returns a \c QImage of the graph
	*/
	QImage GetBitmap(int width, int height, bool keepAspect);
signals:
	void redrawRequired();
protected:
	void mousePressEvent(QGraphicsSceneMouseEvent* ev) override;
private:
	void calculateScale(DataSet& set, double& scaleX, double&scaleY);
	QString getPopupString(const DataItem& item);

	void placeLabel(const QString& label, const QPainterPath& path, const QPointF& optimalPoint, const QBrush& brush);

	bool mDoLabeling;

	DataSet mCurrentData;

	QVector<GraphLayouter*> mLayouters;
	QVector<GraphSorter*> mSorters;

	GraphLayouter* mCurrentLayouter;
	GraphSorter* mCurrentSorter;
	GraphColorizer* mCurrentColorizer;
	GraphLayouter::InterpolationMode mCurrentInterpMode;
	GraphSorter::SortingAttribute mCurrentSortAttribute;
	GraphColorizer::ColoringAttribute mCurrentColoringAttribute;

	QList<QGraphicsItem*> mOldSelection;
private slots:
	void onSelectionChanged();
};

#endif // GRAPHSCENE_H
