#include "stdafx.h"
#include "mainform.h"
#include "fileloader.h"
#include "graphview.h"

struct MainFormPrivate
{
	DataLoader* fileLoader;
	bool fileLoaded;
	DataSet data;
	Ui::MainFormClass ui;
	GraphScene* mGraphScene;
	GraphView* mGraphView;

public:
	MainFormPrivate() :
		fileLoader(nullptr),fileLoaded(false)
	{}
};

MainForm::MainForm(QWidget *parent)
	: QMainWindow(parent), d(*new MainFormPrivate)
{
	d.fileLoader = new DataLoader(this);
	d.ui.setupUi(this);

	QLayout* layout = new QGridLayout(d.ui.graphContainerFrame);
	layout->setMargin(2);

	d.mGraphScene = new GraphScene(this);
	connect(d.mGraphScene,&GraphScene::redrawRequired,this,&MainForm::redrawRequest);

	d.mGraphView = new GraphView(d.mGraphScene,this);
	layout->addWidget(d.mGraphView);

	//fill comboboxes
	auto layouters = d.mGraphScene->GetLayouts();
	for(int i = 0;i<layouters.size();++i)
	{
		d.ui.baseLineComboBox->insertItem(i,layouters.at(i)->GetName());
	}

	d.ui.sortingComboBox->insertItem(0,"Unsorted");
	auto sorters = d.mGraphScene->GetSorters();
	for(int i = 0;i<sorters.size();++i)
	{
		d.ui.sortingComboBox->insertItem(i+1,sorters.at(i)->GetName());
	}

	for(int i=0;i<GraphLayouter::InterpolationMode::SIZE_OF_ENUM;++i)
	{
		d.ui.interpolationComboBox->insertItem(i,GraphLayouter::interpolationMode_str[i]);
	}

	for(int i=0;i<GraphSorter::SortingAttribute::SIZE_OF_ENUM;++i)
	{
		d.ui.sortattributeComboBox->insertItem(i,GraphSorter::sortingAttribute_str[i]);
	}

	for(int i=0;i<GraphColorizer::ColoringAttribute::SIZE_OF_ENUM;++i)
	{
		d.ui.coloringComboBox->insertItem(i,GraphColorizer::coloringAttribute_str[i]);
	}

	//set gradient begin and end colors
	this->updateColorGradient(Qt::yellow, Qt::red);

	this->show();

	QString defFile = "data/boxoffice2013.dat";
	qDebug()<<"Loading default file: "<<defFile;
	loadFile(defFile,false);
}

MainForm::~MainForm()
{
	delete &d;
}

void MainForm::redrawRequest()
{
	if(d.fileLoaded)
	{
		d.mGraphScene->CreateStackedGraph(d.data);
		d.mGraphView->ResetView();
	}
}

bool MainForm::loadFile(const QString& path, bool infoPopup)
{
	d.fileLoaded = d.fileLoader->LoadData(path,d.data);
	if(d.fileLoaded)
	{
		qDebug()<<"Data loaded successfully";	
		if(infoPopup)
			QMessageBox::information(this,tr("Data loaded successfully"),tr("Data file %1 loaded successfully.").arg(path),QMessageBox::Ok);

		d.mGraphScene->CreateStackedGraph(d.data);
		d.mGraphView->ResetView();
	}
	else
	{
		qWarning()<<"Data file could not be loaded due to an invalid or empty file header";
		QMessageBox::critical(this,tr("Error loading data file"),
			tr("There was an error loading the data file %1. Check the log for more information.").arg(path),QMessageBox::Ok);
	}

	return d.fileLoaded;
}

void MainForm::on_btnLoad_clicked()
{
	QString path = QFileDialog::getOpenFileName(this,tr("Select data file to open"),"data",tr("StackedGraph metadata (*.dat)"));

	if(path.isNull())
	{
		qDebug()<<"User canceled file loading";
	}
	else
	{
		qDebug()<<"User selected file "<<QFileInfo(path).absoluteFilePath();

		loadFile(path);
	}
}

void MainForm::on_btnRandom_clicked()
{
	d.fileLoader->RandomData(20,d.data);
	d.mGraphScene->CreateStackedGraph(d.data);
	d.mGraphView->ResetView();
}

QString MainForm::getColorStyleSheet(QVariant const color) const
{
	QString col_str = color.value<QColor>().name();

	return QString("background-color: " + col_str);
}

QString MainForm::getGradientStyleSheet(QVariant const color1, QVariant const color2) const
{
	QString beginColor = color1.value<QColor>().name();
	QString endColor = color2.value<QColor>().name();

	return QString("background-color: qlineargradient(x1:0, y1:0, x2:1, y2:0, stop:0 " + beginColor + ", stop:1 " + endColor + ");");
}

void MainForm::updateColorGradient(QColor const color1, QColor const color2)
{
	if(color1 != nullptr)
	{
		d.ui.gradientLabel->setProperty("Color1",QVariant(color1));
		d.ui.begincolorLabel->setStyleSheet(this->getColorStyleSheet(d.ui.gradientLabel->property("Color1")));
	}
	
	if(color2 != nullptr)
	{
		d.ui.gradientLabel->setProperty("Color2",QVariant(color2));
		d.ui.endcolorLabel->setStyleSheet(this->getColorStyleSheet(d.ui.gradientLabel->property("Color2")));
	}

	if(color1 != nullptr || color2 != nullptr)
	{
		d.ui.gradientLabel->setStyleSheet(this->getGradientStyleSheet(d.ui.gradientLabel->property("Color1"), d.ui.gradientLabel->property("Color2")));
		d.mGraphScene->SetColors(color1, color2);
	}
}

void MainForm::on_begincolorLabel_clicked()
{
	QColor beginColor = QColorDialog::getColor(d.ui.gradientLabel->property("Color1").value<QColor>(),this);
	this->updateColorGradient(beginColor, nullptr);
}

void MainForm::on_endcolorLabel_clicked()
{
	QColor endColor = QColorDialog::getColor(d.ui.gradientLabel->property("Color2").value<QColor>(),this);
	this->updateColorGradient(nullptr, endColor);
}

void MainForm::on_baseLineComboBox_currentIndexChanged(int idx)
{
	d.mGraphScene->SetLayout(d.mGraphScene->GetLayouts().at(idx));
}

void MainForm::on_sortingComboBox_currentIndexChanged(int idx)
{
	//first element in combobox is "unsorted"
	if(idx == 0)
	{
		d.mGraphScene->SetSorter(nullptr);
		d.ui.sortattributeLabel->setEnabled(false);
		d.ui.sortattributeComboBox->setEnabled(false);
	}
	else
	{
		d.ui.sortattributeLabel->setEnabled(true);
		d.ui.sortattributeComboBox->setEnabled(true);
		d.mGraphScene->SetSorter(d.mGraphScene->GetSorters().at(idx-1));
	}
}

void MainForm::on_interpolationComboBox_currentIndexChanged(int idx)
{
	d.mGraphScene->SetInterpolationMode((GraphLayouter::InterpolationMode)idx);
}

void MainForm::on_sortattributeComboBox_currentIndexChanged(int idx)
{
	d.mGraphScene->SetSortingAttribute((GraphSorter::SortingAttribute)idx);
}

void MainForm::on_coloringComboBox_currentIndexChanged(int idx)
{
	d.mGraphScene->SetColoringAttribute((GraphColorizer::ColoringAttribute)idx);
}

void MainForm::on_exportButton_clicked()
{
	QString file = QFileDialog::getSaveFileName(this,tr("Select file to save"),
		"",
		tr("PNG (*.png);;JPEG (*.jpg);;BMP (*.bmp)"));

	if(!file.isNull())
	{
		//render scene
		QImage img = d.mGraphScene->GetBitmap(d.ui.exportWidthSpinBox->value(),d.ui.exportHeigthSpinBox->value(),d.ui.exportAspectCheckBox->isChecked());
		img.save(file);
	}
}

void MainForm::on_labelCheckBox_clicked(bool val)
{
	d.mGraphScene->SetLabeling(val);
}

void MainForm::on_twodcoloringCheckBox_clicked(bool val)
{
	d.mGraphScene->Set2DColoring(val);
}